@@ -151,9 +151,11 @@ void SwitcherData::Thread()
 	std::chrono::milliseconds duration;
 	auto startTime = std::chrono::high_resolution_clock::now();
 	auto endTime = std::chrono::high_resolution_clock::now();
+	switcher->firstIntervalAfterStop = true;
 
 	while (true) {
 		std::unique_lock<std::mutex> lock(m);
+		mainLoopLock = &lock;
 
 		bool match = false;
 		OBSWeakSource scene;
@@ -229,19 +231,6 @@ void SwitcherData::Thread()
 
 		ClearWebsocketMessages();
 
-		// After this point we will call frontend functions like
-		// obs_frontend_set_current_scene() and
-		// obs_frontend_set_current_transition()
-		//
-		// During this time SaveSceneSwitcher() could be called
-		// leading to a deadlock with the frontend function being stuck
-		// in QMetaObject::invokeMethod() holding the mutex and
-		// OBS being stuck in SaveSceneSwitcher().
-		//
-		// So we have to unlock() risking race conditions as these are
-		// less frequent than the above described deadlock
-		lock.unlock();
-
 		if (match) {
 			if (macroMatch) {
 				runMacros();
@@ -251,8 +240,11 @@ void SwitcherData::Thread()
 		}
 
 		writeSceneInfoToFile();
+		switcher->firstInterval = false;
+		switcher->firstIntervalAfterStop = false;
 	}
 
+	mainLoopLock = nullptr;
 	blog(LOG_INFO, "stopped");
 }
 
@@ -280,6 +272,9 @@ void SwitcherData::setPreconditions()
 	}
 	currentTitle = title;
 
+	// Process name
+	GetForegroundProcessName(currentForegroundProcess);
+
 	// Cursor
 	std::pair<int, int> cursorPos = getCursorPos();
 	cursorPosChanged = cursorPos.first != switcher->lastCursorPos.first ||
@@ -420,10 +415,11 @@ void SwitcherData::Start()
 	}
 }
 
-void ResetMacroCounters()
+void ResetMacros()
 {
 	for (auto &m : switcher->macros) {
-		m->ResetCount();
+		m->ResetRunCount();
+		m->ResetTimers();
 	}
 }
 
@@ -440,7 +436,7 @@ void SwitcherData::Stop()
 		th = nullptr;
 
 		writeToStatusFile("Advanced Scene Switcher stopped");
-		ResetMacroCounters();
+		ResetMacros();
 	}
 
 	server.stop();
@@ -598,6 +594,17 @@ void handleExit()
 	FreeSceneSwitcher();
 }
 
+void handleSceneCollectionChanging()
+{
+	if (switcher->settingsWindowOpened) {
+		AdvSceneSwitcher::window->close();
+	}
+	if (!switcher->stop) {
+		switcher->sceneColletionStop = true;
+		switcher->Stop();
+	}
+}
+
 // Note to future self:
 // be careful using switcher->m here as there is potential for deadlocks when using
 // frontend functions such as obs_frontend_set_current_scene()
@@ -643,6 +650,11 @@ static void OBSEvent(enum obs_frontend_event event, void *switcher)
 	case OBS_FRONTEND_EVENT_STREAMING_STOPPING:
 		setStreamStopping();
 		break;
+#if LIBOBS_API_VER >= MAKE_SEMANTIC_VERSION(27, 2, 0)
+	case OBS_FRONTEND_EVENT_SCENE_COLLECTION_CHANGING:
+		handleSceneCollectionChanging();
+		break;
+#endif
 	default:
 		break;
 	}
@@ -676,6 +688,21 @@ void LoadPlugins()
 	}
 }
 
+void OpenSettingsWindow()
+{
+	if (switcher->settingsWindowOpened) {
+		AdvSceneSwitcher::window->show();
+		AdvSceneSwitcher::window->raise();
+		AdvSceneSwitcher::window->activateWindow();
+	} else {
+		AdvSceneSwitcher::window =
+			new AdvSceneSwitcher(static_cast<QMainWindow *>(
+				obs_frontend_get_main_window()));
+		AdvSceneSwitcher::window->setAttribute(Qt::WA_DeleteOnClose);
+		AdvSceneSwitcher::window->show();
+	}
+}
+
 extern "C" void InitSceneSwitcher(obs_module_t *m, translateFunc t)
 {
 	blog(LOG_INFO, "version: %s", g_GIT_TAG);
@@ -689,25 +716,10 @@ extern "C" void InitSceneSwitcher(obs_module_t *m, translateFunc t)
 	LoadPlugins();
 	SetupDock();
 
-	auto cb = []() {
-		if (switcher->settingsWindowOpened) {
-			AdvSceneSwitcher::window->show();
-			AdvSceneSwitcher::window->raise();
-			AdvSceneSwitcher::window->activateWindow();
-		} else {
-			AdvSceneSwitcher::window =
-				new AdvSceneSwitcher(static_cast<QMainWindow *>(
-					obs_frontend_get_main_window()));
-			AdvSceneSwitcher::window->setAttribute(
-				Qt::WA_DeleteOnClose);
-			AdvSceneSwitcher::window->show();
-		}
-	};
-
 	obs_frontend_add_save_callback(SaveSceneSwitcher, nullptr);
 	obs_frontend_add_event_callback(OBSEvent, switcher);
 
 	QAction *action = (QAction *)obs_frontend_add_tools_menu_qaction(
 		obs_module_text("AdvSceneSwitcher.pluginName"));
-	action->connect(action, &QAction::triggered, cb);
+	action->connect(action, &QAction::triggered, OpenSettingsWindow);
 }
