@@ -1,5 +1,6 @@
 #include "advanced-scene-switcher.hpp"
 #include "status-control.hpp"
+#include "file-selection.hpp"
 #include "utility.hpp"
 #include "version.h"
 
@@ -151,6 +152,7 @@ void AdvSceneSwitcher::closeEvent(QCloseEvent *)
 		ui->macroActionConditionSplitter->sizes();
 	switcher->macroListMacroEditSplitterPosition =
 		ui->macroListMacroEditSplitter->sizes();
+
 	obs_frontend_save();
 }
 
@@ -253,11 +255,28 @@ QString getDefaultSaveLocation()
 {
 	QString desktopPath = QStandardPaths::writableLocation(
 		QStandardPaths::DesktopLocation);
+
 	auto scName = obs_frontend_get_current_scene_collection();
 	QString sceneCollectionName(scName);
 	bfree(scName);
+
 	auto timestamp = QDateTime::currentDateTime();
-	return desktopPath + "/adv-ss-" + sceneCollectionName + "-" +
+	auto path = desktopPath + "/adv-ss-" + sceneCollectionName + "-" +
+		    timestamp.toString("yyyy.MM.dd.hh.mm.ss");
+
+	// Check if scene collection name contains invalid path characters
+	QFile file(path);
+	if (file.exists()) {
+		return path;
+	}
+
+	bool validPath = file.open(QIODevice::WriteOnly);
+	if (validPath) {
+		file.remove();
+		return path;
+	}
+
+	return desktopPath + "/adv-ss-" +
 	       timestamp.toString("yyyy.MM.dd.hh.mm.ss");
 }
 
@@ -308,11 +327,8 @@ void AdvSceneSwitcher::on_exportSettings_clicked()
 	}
 
 	obs_data_t *obj = obs_data_create();
-
 	switcher->saveSettings(obj);
-
 	obs_data_save_json(obj, file.fileName().toUtf8().constData());
-
 	obs_data_release(obj);
 }
 
@@ -323,20 +339,20 @@ void AdvSceneSwitcher::on_importSettings_clicked()
 	bool start = !switcher->stop;
 	switcher->Stop();
 
-	std::lock_guard<std::mutex> lock(switcher->m);
-
-	QString directory = QFileDialog::getOpenFileName(
+	auto basePath = FileSelection::ValidPathOrDesktop(
+		QString::fromStdString(switcher->lastImportPath));
+	QString path = QFileDialog::getOpenFileName(
 		this,
 		tr(obs_module_text(
 			"AdvSceneSwitcher.generalTab.saveOrLoadsettings.importWindowTitle")),
-		QDir::currentPath(),
+		basePath,
 		tr(obs_module_text(
 			"AdvSceneSwitcher.generalTab.saveOrLoadsettings.textType")));
-	if (directory.isEmpty()) {
+	if (path.isEmpty()) {
 		return;
 	}
 
-	QFile file(directory);
+	QFile file(path);
 	if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
 		return;
 	}
@@ -350,12 +366,15 @@ void AdvSceneSwitcher::on_importSettings_clicked()
 		return;
 	}
 
+	std::lock_guard<std::mutex> lock(switcher->m);
 	switcher->loadSettings(obj);
-
 	obs_data_release(obj);
+	switcher->lastImportPath = path.toStdString();
 
 	(void)DisplayMessage(obs_module_text(
 		"AdvSceneSwitcher.generalTab.saveOrLoadsettings.loadSuccess"));
+	// Just close the UI and let the user reopen it to not have to
+	// implement updating each setting
 	close();
 
 	// Restart scene switcher if it was active
@@ -496,6 +515,39 @@ void AdvSceneSwitcher::on_tabWidget_currentChanged(int)
 	SetShowFrames();
 }
 
+void AdvSceneSwitcher::on_transitionOverridecheckBox_stateChanged(int state)
+{
+	if (loading) {
+		return;
+	}
+
+	if (!state && !switcher->adjustActiveTransitionType) {
+		DisplayMessage(obs_module_text(
+			"AdvSceneSwitcher.generalTab.transitionBehaviorSelectionError"));
+		ui->adjustActiveTransitionType->setChecked(true);
+	}
+
+	std::lock_guard<std::mutex> lock(switcher->m);
+	switcher->transitionOverrideOverride = state;
+}
+
+void AdvSceneSwitcher::on_adjustActiveTransitionType_stateChanged(int state)
+{
+	if (loading) {
+		return;
+	}
+
+	// This option only makes sense if we are allowed to use transition overrides
+	if (!state && !switcher->transitionOverrideOverride) {
+		DisplayMessage(obs_module_text(
+			"AdvSceneSwitcher.generalTab.transitionBehaviorSelectionError"));
+		ui->transitionOverridecheckBox->setChecked(true);
+	}
+
+	std::lock_guard<std::mutex> lock(switcher->m);
+	switcher->adjustActiveTransitionType = state;
+}
+
 void SwitcherData::loadSettings(obs_data_t *obj)
 {
 	if (!obj) {
@@ -574,7 +626,8 @@ void SwitcherData::saveGeneralSettings(obs_data_t *obj)
 
 	cooldown.Save(obj, "cooldown", "cooldownUnit");
 
-	obs_data_set_bool(obj, "active", !stop);
+	obs_data_set_bool(obj, "active", sceneColletionStop ? true : !stop);
+	sceneColletionStop = false;
 	obs_data_set_int(obj, "startup_behavior", startupBehavior);
 
 	obs_data_set_int(obj, "autoStartEvent",
@@ -599,6 +652,13 @@ void SwitcherData::saveGeneralSettings(obs_data_t *obj)
 	obs_data_set_int(obj, "priority10", functionNamesByPriority[10]);
 
 	obs_data_set_int(obj, "threadPriority", threadPriority);
+
+	obs_data_set_bool(obj, "tansitionOverrideOverride",
+			  transitionOverrideOverride);
+	obs_data_set_bool(obj, "adjustActiveTransitionType",
+			  adjustActiveTransitionType);
+
+	obs_data_set_string(obj, "lastImportPath", lastImportPath.c_str());
 }
 
 void SwitcherData::loadGeneralSettings(obs_data_t *obj)
@@ -674,6 +734,19 @@ void SwitcherData::loadGeneralSettings(obs_data_t *obj)
 	obs_data_set_default_int(obj, "threadPriority",
 				 QThread::NormalPriority);
 	threadPriority = obs_data_get_int(obj, "threadPriority");
+
+	transitionOverrideOverride =
+		obs_data_get_bool(obj, "tansitionOverrideOverride");
+	adjustActiveTransitionType =
+		obs_data_get_bool(obj, "adjustActiveTransitionType");
+
+	if (!transitionOverrideOverride && !adjustActiveTransitionType) {
+		blog(LOG_INFO,
+		     "reset transition behaviour to adjust active transition type");
+		adjustActiveTransitionType = true;
+	}
+
+	lastImportPath = obs_data_get_string(obj, "lastImportPath");
 }
 
 void saveSplitterPos(QList<int> &sizes, obs_data_t *obj, const std::string name)
