@@ -225,7 +225,7 @@ bool MacroActionHotkey::PerformAction()
 
 	if (!keys.empty()) {
 		int dur = _duration;
-		if (_onlySendToObs) {
+		if (_onlySendToObs || !canSimulateKeyPresses) {
 			std::thread t([keys, dur]() { InjectKeys(keys, dur); });
 			t.detach();
 		} else {
@@ -237,12 +237,12 @@ bool MacroActionHotkey::PerformAction()
 	return true;
 }
 
-void MacroActionHotkey::LogAction()
+void MacroActionHotkey::LogAction() const
 {
 	vblog(LOG_INFO, "sent hotkey");
 }
 
-bool MacroActionHotkey::Save(obs_data_t *obj)
+bool MacroActionHotkey::Save(obs_data_t *obj) const
 {
 	MacroAction::Save(obj);
 	obs_data_set_int(obj, "key", static_cast<int>(_key));
@@ -463,9 +463,10 @@ MacroActionHotkeyEdit::MacroActionHotkeyEdit(
 	mainLayout->addLayout(line2Layout);
 	mainLayout->addWidget(_onlySendToOBS);
 	mainLayout->addWidget(_noKeyPressSimulationWarning);
-
 	setLayout(mainLayout);
 
+	_onlySendToOBS->setEnabled(canSimulateKeyPresses);
+
 	_entryData = entryData;
 	UpdateEntryData();
 	_loading = false;
@@ -493,7 +494,8 @@ void MacroActionHotkeyEdit::UpdateEntryData()
 	_leftMeta->setChecked(_entryData->_leftMeta);
 	_rightMeta->setChecked(_entryData->_rightMeta);
 	_duration->setValue(_entryData->_duration);
-	_onlySendToOBS->setChecked(_entryData->_onlySendToObs);
+	_onlySendToOBS->setChecked(_entryData->_onlySendToObs ||
+				   !canSimulateKeyPresses);
 	SetWarningVisibility();
 }
 
