@@ -3,28 +3,34 @@
 #include <QWidget>
 #include <QComboBox>
 
-enum class PluginStateCondition {
-	SCENE_SWITCHED,
-	RUNNING,
-	SHUTDOWN,
-};
-
 class MacroConditionPluginState : public MacroCondition {
 public:
 	MacroConditionPluginState(Macro *m) : MacroCondition(m) {}
 	~MacroConditionPluginState();
 	bool CheckCondition();
-	bool Save(obs_data_t *obj);
+	bool Save(obs_data_t *obj) const;
 	bool Load(obs_data_t *obj);
-	std::string GetId() { return id; };
+	std::string GetId() const { return id; };
 	static std::shared_ptr<MacroCondition> Create(Macro *m)
 	{
 		return std::make_shared<MacroConditionPluginState>(m);
 	}
 
-	PluginStateCondition _condition = PluginStateCondition::SCENE_SWITCHED;
+	enum class Condition {
+		PLUGIN_START,
+		PLUGIN_RESTART,
+		PLUGIN_RUNNING,
+		OBS_SHUTDOWN,
+		SCENE_COLLECTION_CHANGE,
+		PLUGIN_SCENE_CHANGE,
+	};
+
+	Condition _condition = Condition::PLUGIN_SCENE_CHANGE;
 
 private:
+	bool _firstCheckAfterSceneCollectionChange = true;
+
+	const static uint32_t _version;
 	static bool _registered;
 	static const std::string id;
 };
@@ -47,7 +53,7 @@ public:
 	}
 
 private slots:
-	void ConditionChanged(int cond);
+	void ConditionChanged(int idx);
 
 protected:
 	QComboBox *_condition;
