@@ -7,10 +7,10 @@ class MacroConditionProfile : public MacroCondition {
 public:
 	MacroConditionProfile(Macro *m) : MacroCondition(m) {}
 	bool CheckCondition();
-	bool Save(obs_data_t *obj);
+	bool Save(obs_data_t *obj) const;
 	bool Load(obs_data_t *obj);
-	std::string GetShortDesc();
-	std::string GetId() { return id; };
+	std::string GetShortDesc() const;
+	std::string GetId() const { return id; };
 	static std::shared_ptr<MacroCondition> Create(Macro *m)
 	{
 		return std::make_shared<MacroConditionProfile>(m);
