@@ -3,6 +3,7 @@
 #include "macro-condition-edit.hpp"
 #include "macro-action-scene-switch.hpp"
 #include "advanced-scene-switcher.hpp"
+#include "hotkey.hpp"
 
 #include <limits>
 #undef max
@@ -27,8 +28,72 @@ Macro::~Macro()
 	ClearHotkeys();
 }
 
+std::shared_ptr<Macro>
+Macro::CreateGroup(const std::string &name,
+		   std::vector<std::shared_ptr<Macro>> &children)
+{
+	auto group = std::make_shared<Macro>(name, false);
+	for (auto &c : children) {
+		c->SetParent(group);
+	}
+	group->_isGroup = true;
+	group->_groupSize = children.size();
+	return group;
+}
+
+void Macro::RemoveGroup(std::shared_ptr<Macro> group)
+{
+	auto it = std::find(switcher->macros.begin(), switcher->macros.end(),
+			    group);
+	if (it == switcher->macros.end()) {
+		return;
+	}
+
+	auto size = group->GroupSize();
+	for (uint32_t i = 1; i <= size; i++) {
+		auto m = std::next(it, i);
+		(*m)->SetParent(nullptr);
+	}
+
+	switcher->macros.erase(it);
+}
+
+void Macro::PrepareMoveToGroup(Macro *group, std::shared_ptr<Macro> item)
+{
+	for (auto &m : switcher->macros) {
+		if (m.get() == group) {
+			PrepareMoveToGroup(m, item);
+			return;
+		}
+	}
+	PrepareMoveToGroup(std::shared_ptr<Macro>(), item);
+}
+
+void Macro::PrepareMoveToGroup(std::shared_ptr<Macro> group,
+			       std::shared_ptr<Macro> item)
+{
+	if (!item) {
+		return;
+	}
+
+	// Potentially remove from old group
+	auto oldGroup = item->Parent();
+	if (oldGroup) {
+		oldGroup->_groupSize--;
+	}
+
+	item->SetParent(group);
+	if (group) {
+		group->_groupSize++;
+	}
+}
+
 bool Macro::CeckMatch()
 {
+	if (_isGroup) {
+		return false;
+	}
+
 	_matched = false;
 	for (auto &c : _conditions) {
 		if (_paused) {
@@ -113,8 +178,8 @@ bool Macro::CeckMatch()
 	// TODO: Move back to PerformAction() once new scene collection frontend
 	// events are available - see:
 	// https://github.com/obsproject/obs-studio/commit/feda1aaa283e8a99f6ba1159cfe6b9c1f2934a61
-	if (_matched && _count != std::numeric_limits<int>::max()) {
-		_count++;
+	if (_matched && _runCount != std::numeric_limits<int>::max()) {
+		_runCount++;
 	}
 	_lastCheckTime = std::chrono::high_resolution_clock::now();
 	return _matched;
@@ -139,6 +204,10 @@ bool Macro::PerformActions(bool forceParallel, bool ignorePause)
 		RunActions(ret, ignorePause);
 	}
 	_wasExecutedRecently = true;
+	auto group = _parent.lock();
+	if (group) {
+		group->_wasExecutedRecently = true;
+	}
 	return ret;
 }
 
@@ -245,13 +314,29 @@ void Macro::UpdateConditionIndices()
 	}
 }
 
-bool Macro::Save(obs_data_t *obj)
+Macro *Macro::Parent()
+{
+	auto p = _parent.lock();
+	return p.get();
+}
+
+bool Macro::Save(obs_data_t *obj) const
 {
 	obs_data_set_string(obj, "name", _name.c_str());
 	obs_data_set_bool(obj, "pause", _paused);
 	obs_data_set_bool(obj, "parallel", _runInParallel);
 	obs_data_set_bool(obj, "onChange", _matchOnChange);
 
+	obs_data_set_bool(obj, "group", _isGroup);
+	if (_isGroup) {
+		auto groupData = obs_data_create();
+		obs_data_set_bool(groupData, "collapsed", _isCollapsed);
+		obs_data_set_int(groupData, "size", _groupSize);
+		obs_data_set_obj(obj, "groupData", groupData);
+		obs_data_release(groupData);
+		return true;
+	}
+
 	obs_data_set_bool(obj, "registerHotkeys", _registerHotkeys);
 	obs_data_array_t *pauseHotkey = obs_hotkey_save(_pauseHotkey);
 	obs_data_set_array(obj, "pauseHotkey", pauseHotkey);
@@ -332,6 +417,15 @@ bool Macro::Load(obs_data_t *obj)
 	_runInParallel = obs_data_get_bool(obj, "parallel");
 	_matchOnChange = obs_data_get_bool(obj, "onChange");
 
+	_isGroup = obs_data_get_bool(obj, "group");
+	if (_isGroup) {
+		auto groupData = obs_data_get_obj(obj, "groupData");
+		_isCollapsed = obs_data_get_bool(groupData, "collapsed");
+		_groupSize = obs_data_get_int(groupData, "size");
+		obs_data_release(groupData);
+		return true;
+	}
+
 	obs_data_set_default_bool(obj, "registerHotkeys", true);
 	_registerHotkeys = obs_data_get_bool(obj, "registerHotkeys");
 	if (_registerHotkeys) {
@@ -402,6 +496,18 @@ bool Macro::Load(obs_data_t *obj)
 	return true;
 }
 
+bool Macro::PostLoad()
+{
+	ResolveMacroRef();
+	for (auto &c : _conditions) {
+		c->PostLoad();
+	}
+	for (auto &a : _actions) {
+		a->PostLoad();
+	}
+	return true;
+}
+
 void Macro::ResolveMacroRef()
 {
 	for (auto &c : _conditions) {
@@ -594,6 +700,7 @@ void SwitcherData::saveMacros(obs_data_t *obj)
 
 void SwitcherData::loadMacros(obs_data_t *obj)
 {
+	Hotkey::ClearAllHotkeys();
 	switcher->macroProperties.Load(obj);
 
 	macros.clear();
@@ -608,8 +715,41 @@ void SwitcherData::loadMacros(obs_data_t *obj)
 	}
 	obs_data_array_release(macroArray);
 
+	int groupCount = 0;
+	std::shared_ptr<Macro> group;
+	std::vector<std::shared_ptr<Macro>> invalidGroups;
 	for (auto &m : macros) {
-		m->ResolveMacroRef();
+		if (groupCount && m->IsGroup()) {
+			blog(LOG_ERROR,
+			     "nested group detected - will delete \"%s\"",
+			     m->Name().c_str());
+			invalidGroups.emplace_back(m);
+			continue;
+		}
+		if (groupCount) {
+			m->SetParent(group);
+			groupCount--;
+		}
+		if (m->IsGroup()) {
+			groupCount = m->GroupSize();
+			group = m;
+		}
+		m->PostLoad();
+	}
+
+	if (groupCount) {
+		blog(LOG_ERROR,
+		     "invalid group size detected - will delete \"%s\"",
+		     group->Name().c_str());
+		invalidGroups.emplace_back(group);
+	}
+
+	for (auto &m : invalidGroups) {
+		auto it = std::find(macros.begin(), macros.end(), m);
+		if (it == macros.end()) {
+			continue;
+		}
+		macros.erase(it);
 	}
 }
 
@@ -631,11 +771,28 @@ bool SwitcherData::checkMacros()
 
 bool SwitcherData::runMacros()
 {
-	// TODO: Don't rely on creating a copy of each macro once new frontend
-	// events are available - see:
-	// https://github.com/obsproject/obs-studio/commit/feda1aaa283e8a99f6ba1159cfe6b9c1f2934a61
-	for (auto m : macros) {
-		if (m->Matched()) {
+	// Create copy of macor list as elements might be removed, inserted, or
+	// reordered while macros are currently being executed.
+	// For example, this can happen if a macro is performing a wait action,
+	// as the main lock will be unlocked during this time.
+	auto runPhaseMacros = macros;
+
+	// Avoid deadlocks when opening settings window and calling frontend
+	// API functions at the same time.
+	//
+	// If the timing is just right, the frontend API call will call
+	// QMetaObject::invokeMethod(...) with  Qt::BlockingQueuedConnection
+	// while holding the main switcher mutex.
+	// But this invokeMethod call itself will be blocked as it is waiting
+	// the constructor of AdvSceneSwitcher() to complete.
+	// The constructor of AdvSceneSwitcher() cannot continue however as it
+	// cannot lock the main switcher mutex.
+	if (GetLock()) {
+		GetLock()->unlock();
+	}
+
+	for (auto &m : runPhaseMacros) {
+		if (m && m->Matched()) {
 			vblog(LOG_INFO, "running macro: %s", m->Name().c_str());
 			if (!m->PerformActions()) {
 				blog(LOG_WARNING, "abort macro: %s",
@@ -643,6 +800,9 @@ bool SwitcherData::runMacros()
 			}
 		}
 	}
+	if (GetLock()) {
+		GetLock()->lock();
+	}
 	return true;
 }
 
