@@ -1,4 +1,5 @@
 #pragma once
+#include "paramerter-wrappers.hpp"
 
 #include <QDialog>
 #include <QLabel>
@@ -11,21 +12,27 @@
 
 #include <thread>
 #include <mutex>
-
-class MacroConditionVideo;
+#include <condition_variable>
 
 class PreviewDialog : public QDialog {
 	Q_OBJECT
 
 public:
-	PreviewDialog(QWidget *parent, MacroConditionVideo *_conditionData,
-		      std::mutex *mutex);
+	PreviewDialog(QWidget *parent, int delay);
 	virtual ~PreviewDialog();
 	void ShowMatch();
 	void SelectArea();
+	void Stop();
+	void closeEvent(QCloseEvent *event) override;
 
+public slots:
+	void PatternMatchParamtersChanged(const PatternMatchParameters &);
+	void ObjDetectParamtersChanged(const ObjDetectParamerts &);
+	void VideoSelectionChanged(const VideoInput &);
+	void AreaParamtersChanged(const AreaParamters &);
+	void ConditionChanged(int cond);
 private slots:
-	void Resize();
+	void ResizeImageLabel();
 signals:
 	void SelectionAreaChanged(QRect area);
 
@@ -39,7 +46,13 @@ private:
 	void mouseMoveEvent(QMouseEvent *event);
 	void mouseReleaseEvent(QMouseEvent *event);
 
-	MacroConditionVideo *_conditionData;
+	VideoInput _video;
+	PatternMatchParameters _patternMatchParams;
+	PatternImageData _patternImageData;
+	ObjDetectParamerts _objDetectParams;
+	AreaParamters _areaParams;
+
+	VideoCondition _condition = VideoCondition::PATTERN;
 	QScrollArea *_scrollArea;
 	QLabel *_statusLabel;
 	QLabel *_imageLabel;
@@ -49,13 +62,15 @@ private:
 	QRubberBand *_rubberBand = nullptr;
 	std::atomic_bool _selectingArea = {false};
 
-	std::mutex *_mtx;
+	std::mutex _mtx;
+	std::condition_variable _cv;
 	std::thread _thread;
-	std::atomic_bool _stop = {false};
+	std::atomic_bool _stop = {true};
 
 	enum class Type {
 		SHOW_MATCH,
 		SELECT_AREA,
 	};
 	Type _type = Type::SHOW_MATCH;
+	int _delay = 300;
 };
